// -*- C++ -*-
#ifndef RIVET_CDF_2000_S4155203_HH
#define RIVET_CDF_2000_S4155203_HH

#include "Rivet/Analysis.hh"

namespace Rivet {


  /*
   * @brief CDF Run I Z pT in Drell-Yan events
   * @author Hendrik Hoeth
   */ 
  class CDF_2000_S4155203 : public Analysis {
  public:

    /// @name Constructors etc.
    //@{

    /// Constructor: cuts on final state are \f$ -1 < \eta < 1 \f$ 
    /// and \f$ p_T > 0.5 \f$ GeV.
    CDF_2000_S4155203();

    /// Factory method
    static Analysis* create() {
      return new CDF_2000_S4155203();
    }
    //@}


  public:

    string spiresId() const {
      return "4155203";
    }

    string summary() const {
      return "Z pT measurement in CDF Z -> e+ e- events";
    }

    string description() const {
      ostringstream os;
      os << "Measurement of transverse momentum and total cross section of e^+e^- pairs "
         << "in the Z-boson region of $66~\\text{GeV}/c^2 < m_{ee} < 116~\\text{GeV}/c^2$ "
         << "from pbar-p collisions at sqrt(s) = 1.8 TeV, with the Tevatron CDF detector."
         << "\n\n"
         << "The Z pT, in a fully-factorised picture, is generated by the momentum balance "
         << "against initial state radiation (ISR) and the primordial/intrinsic pT of the "
         << "Z's parent partons in the incoming hadrons. The Z pT is important in generator "
         << "tuning to fix the interplay of ISR and multi-parton interactions (MPI) in"
         << "generating `underlying event' activity."
         << "\n\n"
         << "This analysis is subject to ambiguities in the experimental Z pT definition,"
         << "since the Rivet implementation reconstructs the Z momentum from the dilepton"
         << "pair with finite cones for photon brem summation, rather than YFS unfolding"
         << "or a non-portable direct access to the Z in the event record.";
      return os.str();
    }

    string runInfo() const {
      ostringstream os;
      os << "* Tevatron Run I: p pbar collisions at 1800 GeV\n"
         << "* Z Drell-Yan with e+ e- decay mode (for Z \\emph{and} $\\gamma*$) only.";
      return os.str();
    }

    /// Validation status
    string status() const {
      return "VALIDATED";
    }

    string experiment() const {
      return "CDF";
    }

    string collider() const {
      return "Tevatron Run 1";
    }

    string year() const {
      return "2000";
    }

    vector<string> references() const {
      vector<string> rtn;
      rtn += "Phys.Rev.Lett.84:845-850,2000";
      rtn += "arXiv:hep-ex/0001021";
      rtn += "doi:10.1103/PhysRevLett.84.845";
      return rtn;
    }

    vector<string> authors() const {
      vector<string> rtn;
      rtn += "Hendrik Hoeth <hendrik.hoeth@cern.ch>";
      return rtn;
    }


  public:

    /// @name Analysis methods
    //@{
    void init();
    void analyze(const Event& event);
    void finalize();
    //@}


  private:

    AIDA::IHistogram1D *_hist_zpt;

  };


}

#endif
